import 'dart:collection';
import 'dart:math';
import 'package:niralanlayout/data/songs_data.dart';
import 'package:niralanlayout/models/songs_model.dart';
import 'package:niralanlayout/data/sounds.dart';
import 'package:niralanlayout/controllers/settings_controller.dart';
import 'package:audioplayers/audioplayers.dart' hide Logger;
import 'package:flutter/widgets.dart';

class AudioController {
  late AudioCache _musicCache;

  late AudioCache _sfxCache;

  final AudioPlayer _musicPlayer;

  final List<AudioPlayer> _sfxPlayers;

  int _currentSfxPlayer = 0;

  final Queue<SongModel> _playlist;

  final Random _random = Random();

  SettingsController? _settings;

  double sound = 0.5;

  ValueNotifier<AppLifecycleState>? _lifecycleNotifier;

  AudioController({int polyphony = 2})
      : assert(polyphony >= 1),
        _musicPlayer = AudioPlayer(playerId: 'musicPlayer'),
        _sfxPlayers = Iterable.generate(
          polyphony,
          (i) => AudioPlayer(
            playerId: 'sfxPlayer#$i',
            mode: PlayerMode.MEDIA_PLAYER,
          ),
        ).toList(growable: false),
        _playlist = Queue.from(List.of(songsData)..shuffle()) {
    _musicCache = AudioCache(
      fixedPlayer: _musicPlayer,
      prefix: 'assets/music/',
    );
    _sfxCache = AudioCache(
      fixedPlayer: _sfxPlayers.first,
      prefix: 'assets/sfx/',
    );

    _musicPlayer.onPlayerCompletion.listen(_changeSong);
    _musicPlayer.setVolume(sound);
  }

  void attachLifecycleNotifier(
      ValueNotifier<AppLifecycleState> lifecycleNotifier) {
    if (_lifecycleNotifier != null) {
      _lifecycleNotifier!.removeListener(_handleAppLifecycle);
    }
    _lifecycleNotifier = lifecycleNotifier;
    _lifecycleNotifier?.addListener(_handleAppLifecycle);
  }

  void attachSettings(SettingsController settingsController) {
    if (_settings == settingsController) {
      // Already attached to this instance. Nothing to do.
      return;
    }

    if (_settings != null) {
      _settings!.muted.removeListener(_mutedHandler);
      _settings!.musicOn.removeListener(_musicOnHandler);
      _settings!.soundsOn.removeListener(_soundsOnHandler);
    }

    _settings = settingsController;

    _settings!.muted.addListener(_mutedHandler);
    _settings!.musicOn.addListener(_musicOnHandler);
    _settings!.soundsOn.addListener(_soundsOnHandler);

    if (!_settings!.muted.value && _settings!.musicOn.value) {
      _startMusic();
    }
  }

  void dispose() {
    _lifecycleNotifier?.removeListener(_handleAppLifecycle);
    _stopAllSound();
    _musicPlayer.dispose();
    for (final player in _sfxPlayers) {
      player.dispose();
    }
  }

  void initialize() async {
    await _sfxCache
        .loadAll(SfxType.values.expand(soundTypeToFilename).toList());
  }

  void soundsOnHandler() {
    for (final player in _sfxPlayers) {
      if (player.state == PlayerState.PLAYING) {
        player.stop();
      }
    }
  }

  void playSfx(SfxType type) {
    final muted = _settings?.muted.value ?? true;
    if (muted) {
      return;
    }
    final soundsOn = _settings?.soundsOn.value ?? false;
    if (!soundsOn) {
      return;
    }

    final options = soundTypeToFilename(type);
    final filename = options[_random.nextInt(options.length)];
    _sfxCache.play(filename, volume: soundTypeToVolume(type));
    _currentSfxPlayer = (_currentSfxPlayer + 1) % _sfxPlayers.length;
    _sfxCache.fixedPlayer = _sfxPlayers[_currentSfxPlayer];
  }

  void _changeSong(void _) {
    _playlist.addLast(_playlist.removeFirst());
    _musicCache.play(_playlist.first.filename);
  }

  void _handleAppLifecycle() {
    switch (_lifecycleNotifier!.value) {
      case AppLifecycleState.paused:
      case AppLifecycleState.detached:
        _stopAllSound();
        break;
      case AppLifecycleState.resumed:
        if (!_settings!.muted.value && _settings!.musicOn.value) {
          _resumeMusic();
        }
        break;
      case AppLifecycleState.inactive:
        break;
    }
  }

  void _musicOnHandler() {
    if (_settings!.musicOn.value) {
      if (!_settings!.muted.value) {
        _resumeMusic();
      }
    } else {
      _stopMusic();
    }
  }

  void _mutedHandler() {
    if (_settings!.muted.value) {
      _stopAllSound();
    } else {
      if (_settings!.musicOn.value) {
        _resumeMusic();
      }
    }
  }

  void _resumeMusic() async {
    switch (_musicPlayer.state) {
      case PlayerState.PAUSED:
        try {
          await _musicPlayer.resume();
        } catch (e) {
          _musicCache.play(_playlist.first.filename);
          _musicPlayer.setVolume(sound);
        }
        break;
      case PlayerState.STOPPED:
        _musicCache.play(_playlist.first.filename);
        break;
      case PlayerState.PLAYING:
        _musicPlayer.setVolume(sound);
        break;
      case PlayerState.COMPLETED:
        _musicCache.play(_playlist.first.filename);
        _musicPlayer.setVolume(sound);
        break;
    }
  }

  void _soundsOnHandler() {
    for (final player in _sfxPlayers) {
      if (player.state == PlayerState.PLAYING) {
        player.stop();
      }
    }
  }

  void _startMusic() {
    _musicCache.play(_playlist.first.filename);
  }

  void _stopAllSound() {
    if (_musicPlayer.state == PlayerState.PLAYING) {
      _musicPlayer.pause();
    }
    for (final player in _sfxPlayers) {
      player.stop();
    }
  }

  void _stopMusic() {
    if (_musicPlayer.state == PlayerState.PLAYING) {
      _musicPlayer.pause();
    }
  }
}
